// Aml2Pic.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <commdlg.h>
#include <winspool.h>

HMODULE m_hModule;

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  r, 
                       LPVOID /*lpReserved*/
					 )
{
	if (DLL_PROCESS_ATTACH==r)
		m_hModule=(HMODULE)hModule;
    return TRUE;
}

// GetPrinterDC()
// returns a printer DC - uses Printer Common Dialog
HDC GetPrinterDC(BOOL& bCanceled)
{
    PRINTDLG pdlg={0};

    pdlg.lStructSize = sizeof( PRINTDLG );
    pdlg.Flags =  PD_RETURNDC;// PD_RETURNDEFAULT
    bCanceled=!PrintDlg( &pdlg );
    return pdlg.hDC;
}

// Get the length, in characters, of the text in the control
int GetRTFTextLength(const HWND hWndRTF )
{
    GETTEXTLENGTHEX gtlex = { GTL_PRECISE, CP_ACP };
    return SendMessage( hWndRTF, EM_GETTEXTLENGTHEX, (WPARAM)&gtlex, 0 );
}

// RTFToEMF - Tell the control to draw itself on the EMF
// Parameters:
//     hRefDC is used to create the EMF
//     pszMetaFileName is the file name of the new EMF (can be NULL)
//     prcMeta is the RECT used to in CreateEnhMetaFile(), in 0.01mm
//          units (should not be NULL)
//     hWndRTF is the control of interest
//     nStart is the starting character location
//     *pEnd is a pointer to an int which receives the position of
//          the next character to print after this page (can be NULL)
HENHMETAFILE RTFToEMF( HDC hRefDC, LPCTSTR pszMetaFileName, LPCRECT prcMeta,
                      const HWND hWndRTF, int nStart, int *pEnd )
{
    HDC             hMetaDC;
    FORMATRANGE     fr;
    int             nTextPrinted;

    // Create the EMF
    hMetaDC = CreateEnhMetaFile( hRefDC, pszMetaFileName, prcMeta, NULL );
    if( hMetaDC == NULL )
        return NULL;

    ZeroMemory(&fr, sizeof(fr));
    // Set up the page (convert 0.01mm to twips)
    fr.rcPage.top       = prcMeta->left*1440/2540;
    fr.rcPage.left      = prcMeta->top*1440/2540;
    fr.rcPage.right     = prcMeta->right*1440/2540;
    fr.rcPage.bottom    = prcMeta->bottom*1440/2540;
    // Set up no margins all around.
    fr.rc = fr.rcPage;
    // Set up the range of text to print as nStart to end of document
    fr.chrg.cpMin = nStart;
    fr.chrg.cpMax = -1;
    fr.hdc = fr.hdcTarget = hMetaDC;
    // Tell the control to draw itself on our (meta) DC
    nTextPrinted = SendMessage(hWndRTF, EM_FORMATRANGE, TRUE, (LPARAM)&fr);
    if( pEnd != NULL )
        *pEnd = nTextPrinted;
    return CloseEnhMetaFile( hMetaDC );
}

// DumpRTFToPagedEMFs - demonstrates using RTFToEMF() to create an EMF
//                      for each page in an RTF control
// Parameters:
//     hWndRTFControl - the control
//     szEMFFileTitleBase - base filename for EMF files, number is appended
void DumpRTFToPagedEMFs( HWND hWndRTFControl, LPTSTR szEMFFileTitleBase ,const HDC hRefDC)
{
    TCHAR           szMetaName[MAX_PATH];
    int             nRTFTextLength, nStart, nPage;
    //HDC             hRefDC;
    RECT            rcMeta;
    HENHMETAFILE    hEMF;

    // First, determine how many chars are in the RTF
    nRTFTextLength = GetRTFTextLength( hWndRTFControl );
    // Get a reference DC (based on a printer)
    //hRefDC = GetPrinterDC();
    // Set up the meta RECT for 0.01mm units
    SetRect( &rcMeta, 0, 0, GetDeviceCaps(hRefDC, HORZSIZE)*100, 
                            GetDeviceCaps(hRefDC, VERTSIZE)*100 );

    // Loop while we've not reached the end of the text in the control
    for(nPage=0,nStart=0;nStart<nRTFTextLength;)
    {
        // construct a file name for this page
        wsprintf(szMetaName, TEXT("%s%d.EMF"), szEMFFileTitleBase, nPage);
        // call function above to draw this portion of the RTF on the EMF
        hEMF = RTFToEMF( hRefDC, szMetaName, &rcMeta, hWndRTFControl,
                         nStart, &nStart );
        // clean up
        DeleteEnhMetaFile( hEMF );
        nPage++;
    }

	//const BOOL b=DeleteDC(hRefDC);
} 

extern "C"
BOOL __declspec(dllexport) Aml_GetPluginInfo(AML_PLUGIN_INFO* p)
{
	strcpy(p->szMenuName,"Aml2Pic");
	p->nMenuCount=1;
	return TRUE;
}

extern "C"
BOOL __declspec(dllexport) Aml_GetMenuItemInfo(int /*nIndex*/,AML_MENU_ITEM_INFO* p)
{
	strcpy(p->szFunctionName,"Aml2PicProc");
	strcpy(p->szMenuName,"Save Text To Picture Files...");
	p->nPluginType=APPT_PAGECONTENT;

	return TRUE;
}


class CWaitCursor {//auto hourglass cursor
	const HCURSOR hPrevCursor;
public:
	CWaitCursor():hPrevCursor(SetCursor(LoadCursor(NULL,IDC_WAIT)))
	{
	}
	~CWaitCursor()
	{
		SetCursor(hPrevCursor);
	}
};

extern "C"
BOOL __declspec(dllexport) Aml2PicProc(AML_PAGECONTENT* p)
{
	BOOL bCanceledPrinterSelect=FALSE;
	const HDC hdcPrinter=GetPrinterDC(bCanceledPrinterSelect);
	if (!hdcPrinter) {
		if (!bCanceledPrinterSelect) {
			MessageBox(p->hwnd,"Plugin required installed printer.",NULL,MB_ICONEXCLAMATION);
			return FALSE;
		}//if (!bCanceledPrinterSelect) {
		else
			return TRUE;
	}//if (!hdcPrinter) {
	
	OPENFILENAME ofn={0};
	ofn.lStructSize=sizeof(ofn);
	ofn.hwndOwner=APPHelper::GetMainHWND();
	ofn.hInstance=m_hModule;
	ofn.hwndOwner=APPHelper::GetMainHWND();//  =  Aml Pages
	ofn.lpstrFilter="Enhanced Metafiles (*.emf)\0*.emf\0All Files (*.*)\0*.*\0\0";
	ofn.Flags=OFN_OVERWRITEPROMPT|OFN_HIDEREADONLY|OFN_CREATEPROMPT;
	
	ofn.lpstrTitle="Save Text To Picture Files";
	ofn.nMaxFileTitle=strlen(ofn.lpstrTitle);
	
	TCHAR szFNAME[MAX_PATH+1]={0};
	ofn.lpstrFile=szFNAME;
	ofn.nMaxFile=sizeof(szFNAME);	

	const int nPageID=APPHelper::GetActivePageID();
	const int nBytes=APPHelper::GetPageHeaderBytesNeed(nPageID);
	
	if (nBytes>0 && nBytes<sizeof(szFNAME)/sizeof(TCHAR)) {
		APPHelper::GetPageHeader(nPageID,ofn.lpstrFile);
	}

	if (TRUE == GetSaveFileName(&ofn)) {
		CWaitCursor wait_cursor;
		DumpRTFToPagedEMFs(p->hwnd,ofn.lpstrFile,hdcPrinter);
	}	

	return TRUE;
}
